<?php

/**
 * @file
 * Implementation of Twitter DB API methods.
 */
define('TWITTER_DB_TABLE', 'twitter_db');
define('TWITTER_DB_URL', 'twitter.com');
define('TWITTER_DB_SETTINGS', 'admin/config/media/twitter-db');

/**
 * Update the Twitter DB table with new tweets. Only tweets newer than last update will be saved
 */
function _twitter_db_update() {
  $retrieve_name = variable_get('twitter_db_retrieve_name', '');
  $type = variable_get('twitter_db_what_to_retrieve', 'user');
  if (!$retrieve_name) {
    drupal_set_message(t('Twitter DB: No Twitter account or hashtag is set. New tweets can not be retrieved.'), 'warning');
    return;
  }
  switch ($type) {
    case 'user':
      $rss_url = "http://twitter.com/statuses/user_timeline/{$retrieve_name}.xml";
      break;
    case 'hashtag':
      $rss_url = "http://search.twitter.com/search.rss?q={$retrieve_name}";
      break;
  }
  $rss = drupal_http_request($rss_url);
  if ($rss->code == 200) {
    $xml = simplexml_load_string($rss->data);
    if ($xml !== false) {
      switch ($type) {
        case 'user':
          $tweets = _twitter_db_parse_user_timeline($xml);
          break;

        case 'hashtag':
          $tweets = _twitter_db_parse_hashtag($xml);
          break;
      }
      if ($tweets) {
        drupal_set_message(t('The Twitter DB table has been updated!'));
      }
    }
  }
}

/**
 * Get the $quantity last tweets. If $quantity is 0, then all tweets is returned.
 * @param integer $quantity The number of tweets to return. If 0, all tweets are returned.
 * @return DatabaseStatementBase Object 
 */
function _twitter_db_get_last_tweets($quantity = 0) {
  if ($quantity > 0) {
    $result = db_select(TWITTER_DB_TABLE, 't')
        ->fields('t')
        ->orderBy('created', 'DESC')
        ->range(0, $quantity)
        ->execute();
  }
  else {
    $result = db_select(TWITTER_DB_TABLE, 't')
        ->fields('t')
        ->orderBy('created', 'DESC')
        ->execute();
  }
  return $result;
}

/**
 * Returns the content to show in a block or pane.
 * @return string HTML content with the last tweets.
 */
function _twitter_db_generate_content() {
  $output = '';
  $content = '';
  $quantity = variable_get('twitter_db_number_of_tweets_to_display', 3);
  $tweets = _twitter_db_get_last_tweets($quantity);
  $title = '';
  $type = 'ul';
  $attributes = array(
    'id' => 'twitter-db-list',
    'class' => 'tweet-list',
  );
  $items = array();
  foreach ($tweets as $record) {
    $item_content = theme('twitter_db_tweet', array(
      'id' => $record->id,
      'tid' => $record->tid,
      'created' => _twitter_db_relative_time($record->created),
      'text' => _twitter_db_replaces($record->text, $record->type),
      'source' => $record->source,
      'replyto' => $record->replyto,
      'type' => $record->type,
      'user_id' => $record->user_id,
      'user_name' => $record->user_name,
      'user_profile_pic' => $record->user_profile_pic,
        ));
    $items[] = array(
      'data' => $item_content,
      'id' => $record->id,
      'class' => '',
    );
  }

  $content .= theme_item_list(array(
    'items' => $items,
    'title' => $title,
    'type' => $type,
    'attributes' => $attributes
      ));

  $output .= theme('twitter_db_tweets', array('tweets' => $content));

  return $output;
}

/**
 * Batch for Twitter DB update
 */
function _twitter_db_update_batch() {
  $batch = array(
    'operations' => array(
      array('_twitter_db_update_process'),
      ),
    'finished' => '_twitter_db_update_finished',
    'title' => t('Updating Twitter DB table'),
    'init_message' => t('Twitter DB update is starting.'),
    'progress_message' => t('Processed @current out of @total.'),
    'error_message' => t('Twitter DB update has encountered an error.'),
  );
  batch_set($batch);

  // If this function was called from a form submit handler, stop here,
  // FAPI will handle calling batch_process().

  // If not called from a submit handler, add the following,
  // noting the url the user should be sent to once the batch
  // is finished.
  batch_process('node/1');  
}


/**
 * Batch 'processed' callback
 */
function _twitter_db_update_process(&$context) {
  
  
//  _twitter_db_update_table();
  
  
  watchdog('Twitter', print_r($context, TRUE));
  
  
  
  if (!isset($context['sandbox']['progress'])) {
    $context['sandbox']['progress'] = 0;
    $context['sandbox']['current_node'] = 0;
    $context['sandbox']['max'] = 20;
  }

  // For this example, we decide that we can safely process
  // 5 nodes at a time without a timeout.
  $limit = 5;

  // With each pass through the callback, retrieve the next group of nids.
  $result = db_query_range("SELECT nid FROM {node} WHERE nid > %d ORDER BY nid ASC", $context['sandbox']['current_node'], 0, $limit);
  while ($row = db_fetch_array($result)) {

    // Here we actually perform our processing on the current node.
    $node = node_load($row['nid'], NULL, TRUE);
    $node->value1 = $options1;
    $node->value2 = $options2;
    node_save($node);

    // Store some result for post-processing in the finished callback.
    $context['results'][] = check_plain($node->title);

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['sandbox']['current_node'] = $node->nid;
    $context['message'] = t('Now processing %node', array('%node' => $node->title));
  }

  // Inform the batch engine that we are not finished,
  // and provide an estimation of the completion level we reached.
  if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
    $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
  }
  
  
  
}

/**
 * Batch 'finished' callback
 */
function _twitter_db_update_finished($success, $results, $operations) {
  if ($success) {
    $message = count($results) .' processed.';
    $message .= theme('item_list', $results);
  }
  else {
    // An error occurred.
    // $operations contains the operations that remained unprocessed.
    $error_operation = reset($operations);
    $message = t('An error occurred while processing %error_operation with arguments: @arguments', array('%error_operation' => $error_operation[0], '@arguments' => print_r($error_operation[1], TRUE)));
  }
  drupal_set_message($message);
  
  $destination = drupal_get_destination();
  if ($destination) {
    drupal_goto($destination);
  }
  else {
    drupal_goto(TWITTER_DB_SETTINGS);
  }
}

/**
 * Trucate the Twitter DB table and reset update status. All the last tweets will be downloaded again.
 * Note that only the last 20 tweets will be retrieved, which means that older tweets can be deleted
 * from the database.
 */
function _twitter_db_update_table() {
  variable_set('twitter_db_last_update', 0);
  $result = db_truncate(TWITTER_DB_TABLE)->execute();
  _twitter_db_update();
  $destination = drupal_get_destination();
  if ($destination) {
    drupal_goto($destination);
  }
  else {
    drupal_goto(TWITTER_DB_SETTINGS);
  }
}

/**
 * Replaces URLs and user names in the tweet with links for $type = 'user'.
 * @param string $text The Twitter text.
 * @param string $type The of tweet.
 * @return string The tweet text, parsed or not.
 */
function _twitter_db_replaces($text, $type) {
  if ($type == 'user') {
    $output = $text;
    // Replaces URLs with a link
    $pattern_url = "/\b((?:[a-z][\w-]+:(?:\/{1,3}|[a-z0-9%])|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}\/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'\".,<>?«»“”‘’]))/i";
    $replacement_url = '<a href="$1">$1</a>';
    $output = preg_replace($pattern_url, $replacement_url, $output);
    // Replaces Twitter user names with a link to the user
    $pattern_user = "/(^|[\W])@(\w+)/i";
    $replacement_user = '$1@<a href="http://' . TWITTER_DB_URL . '/$2">$2</a>';
    $output = preg_replace($pattern_user, $replacement_user, $output);
    return $output;
  }
  return $text;
}

/**
 * Set att a job to be executed every $period.
 * @param integer $period The period in timestamp.
 */
function _twitter_db_set_job($period = 3600) {
  $job = array(
    'type' => 'story',
    'id' => 12,
    'period' => $period,
    'periodic' => TRUE,
  );
  JobScheduler::get('twitter_db_update')->set($job);
}

/**
 * Return the relative time to $date, in other words NOW - $date.
 * @param integer $date Date and time in timestamp format.
 * @return string The relative time between NOW and $date, ex "2 hours ago".
 */
function _twitter_db_relative_time($date) {
  $relative_to = (strlen($date) > 1) ? $date : time();
  $delta = (time() - $relative_to);
  if ($delta < 1) {
    $r = t('Right now');
  }
  else if ($delta < 60) {
    $r = round($delta) . ' ' . t('seconds ago');
  }
  else if ($delta < 120) {
    $r = ' ' . t('1 minute ago');
  }
  else if ($delta < (45 * 60)) {
    $r = round($delta / 60) . ' ' . t('minutes ago');
  }
  else if ($delta < (2 * 60 * 60)) {
    $r = ' ' . t('1 hour ago');
  }
  else if ($delta < (24 * 60 * 60)) {
    $r = round($delta / 3600) . ' ' . t('hours ago');
  }
  else if ($delta < (48 * 60 * 60)) {
    $r = ' ' . t('1 day ago');
  }
  else {
    $r = round($delta / 86400) . ' ' . t('days ago');
  }
  return $r;
}

/**
 * Parse a SimpleXML object with a user timeline and store the tweets in the database.
 * @param SimpleXMLObject $tweets The tweets in SimpleXML format.
 * @return boolean TRUE or FALSE.
 */
function _twitter_db_parse_user_timeline($tweets) {
  $last_update = variable_get('twitter_db_last_update', 0);
  foreach ($tweets->status as $tweet) {
    $created = strtotime($tweet->created_at);
    // Get tweet only if it is newer than the last update
    if ($created > $last_update) {
      try {
        $postid = db_insert(TWITTER_DB_TABLE)
            ->fields(array(
              'tid' => $tweet->id,
              'created' => $created,
              'text' => $tweet->text,
              'source' => $tweet->source,
              'replyto' => $tweet->in_reply_to_screen_name,
              'type' => 'user',
              'user_id' => $tweet->user->id,
              'user_name' => $tweet->user->name,
              'user_profile_pic' => $tweet->user->profile_image_url,
            ))
            ->execute();
      } catch (Exception $e) {
        watchdog('Twitter DB: update', $e->getMessage());
      }
    }
  }
  variable_set('twitter_db_last_update', time());
  return TRUE;
}

/**
 * Parse a SimpleXML object with a hashtag and store the tweets in the database.
 * @param SimpleXMLObject $tweets The tweets in SimpleXML format.
 * @return boolean TRUE or FALSE.
 */
function _twitter_db_parse_hashtag($tweets) {
  $last_update = variable_get('twitter_db_last_update', 0);
  foreach ($tweets->channel->item as $tweet) {
    $created = strtotime($tweet->pubDate);
    // Get tweet only if it is newer than the last update
    if ($created > $last_update) {
      try {
        $urlvars = explode('/', $tweet->link);
        $tid = $urlvars[5];
        $user_name = $urlvars[3];
        $user = _twitter_db_get_user_profile($user_name);
        if ($user) {
          $user_id = $user->id;
          $user_profile_pic = $user->profile_image_url;
        }
        else {
          $user_id = '';
          $user_profile_pic = '';
        }
        $postid = db_insert(TWITTER_DB_TABLE)
            ->fields(array(
              'tid' => $tid,
              'created' => $created,
              'text' => $tweet->description,
              'source' => '',
              'replyto' => '',
              'type' => 'hashtag',
              'user_id' => $user_id,
              'user_name' => $user_name,
              'user_profile_pic' => $user_profile_pic,
            ))
            ->execute();
      } catch (Exception $e) {
        watchdog('Twitter DB: update', $e->getMessage());
      }
    }
  }
  variable_set('twitter_db_last_update', time());
  return TRUE;
}

/**
 * Get and return  an user object from Twitter.
 * @param string $screen_name The user's screen name.
 * @return SimpleXMLObject/boolean The user object or FALSE if it fails.
 */
function _twitter_db_get_user_profile($screen_name) {
  $url = "https://api.twitter.com/1/users/show.xml?screen_name={$screen_name}&include_entities=true";
  $rss = drupal_http_request($url);
  if ($rss->code == 200) {
    $xml = simplexml_load_string($rss->data);
    if ($xml !== false) {
      return $xml;
    }
    else {
      return FALSE;
    }
  }
}

